# -*- coding: utf-8 -*-
'''
Django.js provide better integration of javascript into Django.
'''

__version__ = '0.7.4'
__description__ = "Django JS Tools"

#: Packaged jQuery version
JQUERY_DEFAULT_VERSION = '2.0.0'
JQUERY_MIGRATE_VERSION = '1.1.1'
