{% extends "djangojs/jasmine-runner.html" %}

{% load js %}
{% load url from future %}

{% block js_content %}
    {% include "djangojs/test/test-data.html" %}
{% endblock %}

{% block body_content %}
  <form id="test-form" action="{% url "test_form" %}" method="POST" style="display: none;">
    <span id="csrf-token">{{csrf_token}}</span>
    {{form}}
  </form>
  <form id="test-csrf-elem" action="{% url "test_form" %}" method="POST" style="display: none;">
    {% csrf_token %}
    {{form}}
  </form>
{% endblock %}
