/**
 * QUnit Ninja Theme by Jamie Hoover
 *
 * http://qunitjs.com
 *
 * Copyright 2012 jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */
body {
	background:#111 no-repeat -webkit-gradient(linear, left top, left bottom, color-stop(0, #222), color-stop(1, #111));
	background:#111 no-repeat -webkit-linear-gradient(top, #222 0%, #111 100%);
	background:#111 no-repeat -moz-linear-gradient(top, #222 0%, #111 100%);
	background:#111 no-repeat -o-linear-gradient(top, #222 0%, #111 100%);
	background:#111 no-repeat -ms-linear-gradient(top, #222 0%, #111 100%);
	background:#111 no-repeat linear-gradient(top, #222 0%, #111 100%);
	color:#ccc;
	font:12px/16px "Lucida Grande",sans-serif;
	margin:0 auto;
	max-width:1024px;
	min-width:800px;
	padding:24px;
	position:relative;
}
#qunit-header {
	font-size:24px;
	line-height:1;
	margin:0 520px 0 0;
}
#qunit-header > a {
	color:#ccc;
	text-decoration:none;
}
#qunit-header > a:hover {
	color:#ccc;
	text-decoration:underline;
}
#qunit-testresult {
	font:12px/12px "Menlo",monospace;
	margin:0;
	position:absolute;
	right:24px;
	top:20px;
}
#qunit-testresult .module-name {
	font-weight:bold;
}
#qunit-testresult br {
	content:'';
}
#qunit-testresult span:before {
	content:' ';
}
#qunit-testresult span.failed {
	color:#c33;
}
#qunit-testresult span.passed {
	color:#6c6;
}
#qunit-testrunner-toolbar {
	position:absolute;
	right:24px;
	top:40px;
}
#qunit-testrunner-toolbar input {
	font:inherit;
	margin:0 6px 0 24px;
}
#qunit-testrunner-toolbar label {
	display:inline-block;
}
#qunit-testrunner-toolbar select {
	position:absolute;
	right:24px;
	top:22px;
}
#qunit-modulefilter-container label {
	display:none;
}
#qunit-banner {
	height:1px;
	margin:24px 0;
}
#qunit-banner.qunit-fail {
	background-color:#c33;
}
#qunit-banner.qunit-pass {
	background-color:#6c6;
}
#qunit-tests {
	color:#666;
	font:12px/24px "Menlo",monospace;
	list-style-position:inside;
	padding:0;
}
#qunit-tests b.counts {
	font-weight:normal;
}
#qunit-tests b.counts b {
	font-weight:normal;
}
#qunit-tests.hidepass li.pass,#qunit-tests.hidepass li.running {
	display:none;
}
#qunit-tests li a {
	color:#666;
	margin-left:6px;
	text-decoration:none;
}
#qunit-tests li a:hover {
	color:#ccc;
}
#qunit-tests li a:hover,#qunit-tests li strong:hover {
	text-decoration:underline;
}
#qunit-tests li b.counts {
	color:#666;
}
#qunit-tests li.fail span.test-message {
	color:#c33;
}
#qunit-tests li.fail strong {
	color:#c33;
}
#qunit-tests li.fail strong b.counts b.failed {
	color:#c33;
}
#qunit-tests li.pass b.passed {
	color:#6c6;
}
#qunit-tests li.pass span.test-message {
	color:#6c6;
}
#qunit-tests li.pass strong {
	color:#ccc;
}
#qunit-tests li strong {
	font-weight:normal;
}
#qunit-tests li strong:hover {
	cursor:pointer;
}
#qunit-tests ol {
	list-style-position:inside;
	padding:0 0 0 22px;
}
#qunit-tests ol li table {
	border-collapse:collapse;
}
#qunit-tests ol li table td pre {
	background-color:#222;
	-webkit-border-radius:4px;
	border-radius:4px;
	color:#ccc;
	font:inherit;
	margin:3px 0 3px 12px;
	padding:12px 24px;
	white-space:pre-wrap;
	word-wrap:break-word;
}
#qunit-tests ol li table th {
	font-weight:normal;
}
#qunit-tests ol li table th,#qunit-tests ol li table td {
	padding:0;
	vertical-align:top;
}
#qunit-tests ol li table tr.test-actual {
	color:#c33;
}
#qunit-tests ol li table tr.test-diff del {
	color:#6c6;
}
#qunit-tests ol li table tr.test-diff del,#qunit-tests ol li table tr.test-diff ins {
	text-decoration:none;
}
#qunit-tests ol li table tr.test-diff ins {
	color:#c33;
}
#qunit-tests ol li table tr.test-expected {
	color:#6c6;
}
#qunit-tests ol li table tr.test-source pre {
	padding:12px 24px 12px 0;
}
#qunit-userAgent {
	color:#666;
	font:9px/9px "Menlo",monospace;
}
#qunit-fixture {
	position:absolute;
	height:768px;
	left:-1024px;
	top:-768px;
	width:1024px;
}
